clear variables
close all

% Physical properties _____________________________________________________
L   = 1.0;      % Pendulum length
m   = 2.0;      % Point mass at the end of the pendulum
g   = 9.81;     % Gravity

% Coordinates _____________________________________________________________

% Selected x, y, and z coordinates of the pendulum tip
% Initial configuration
x   = 0.0;      y  = 0.0;        z = -L;
xd  = 0.0;      yd = 0.0;       zd = 0.0;
q   = [x; y; z];
qd  = [xd; yd; zd];
qdd = [0.0; 0.0; 0.0];

theta   = 0.0;
phi     = 0.0;
cth     = cos(theta);
sth     = sin(theta);
cph     = cos(phi);
sph     = sin(phi);

tic
for i=1:10000
% Dynamic terms ___________________________________________________________

    % Mass matrix (constant)
    M   = m*eye(3);

    % Forces vector
    Q   = [0.0; 0.0; -m*g];

    % Constraints Jacobian
    A   = [2.0 2.0 2.0];

    % Velocity transformation matrix
    R   = L*[   cth*cph, -sth*sph;
                cth*sph,  sth*cph;
                sth,        0];

    % Partial derivative of R w.r.t. independent velocities
    dRTdz(:,:,1) = L*[  -sth*cph, -sth*sph, cth;
                        -cth*sph,  cth*cph, 0.0];
    dRTdz(:,:,2) = L*[  -cth*sph, -cth*cph, 0.0;
                        -sth*cph, -sth*sph, 0.0];

    % Linearization terms _____________________________________________________
    Mr = R'*M*R;
    Kr = -[dRTdz(:,:,1) * Q, dRTdz(:,:,2) * Q];

    Znn     = zeros(2);
    Er      = [Mr, Znn; Znn, eye(2)];
    Ar      = [Znn, Kr; -eye(2), Znn];

    lambda  = eig(-Ar, Er);


    % Compare to analytical solution __________________________________________
    error_check = zeros(2,1);
    j = 1;
    for k=1:4
        if (isfinite(lambda(k)) == 1)
            error_check(j) = sqrt( lambda(k) - (-1)^(k+1)*sqrt(-g/L) )^2; 
            j = j+1;
        end
    end
end
toc

norm(error_check,2)
    